<?php

/**
 * @file
 * FeedsDrupalUserLdapEntryFetcher.
 */

use Drupal\Core\Url;

/**
 *
 */
class FeedsDrupalUserLdapEntryFetcherResult extends FeedsFetcherResult {

  public $filterLdapAuthenticated;
  public $availableDrupalUserAttributes;
  public $filterRoles;

  /**
   * Constructor.
   */
  public function __construct($source_config) {
    $this->availableDrupalUserAttributes = ldap_feeds_drupal_user_attributes();
    $this->filterLdapAuthenticated = isset($source_config['filterLdapAuthenticated']) ? $source_config['filterLdapAuthenticated'] : FALSE;
    $this->filterRoles = isset($source_config['filterRoles']) ? $source_config['filterRoles'] : FALSE;

    parent::__construct('');
    $this->ldap_result = $this->getRaw();
    return $this->ldap_result;
  }

  /**
   * Overrides parent::getRaw();.
   */
  public function getRaw() {

    // Needs to loop through all users, and query ldap for each, one at a time.
    $query = new EntityFieldQuery();
    $entities = $query
      ->entityCondition('entity_type', 'user')
      ->execute();
    $users = entity_load('user', array_keys($entities['user']));
    if ($this->filterRoles) {
      $selectedRoles = array_filter($this->filterRoles);
      $filterOnRoles = (boolean) (count($selectedRoles));
    }
    else {
      $filterOnRoles = FALSE;
    }

    foreach ($users as $uid => $user) {
      if (
        $uid == 0 ||
        $uid == 1 ||
        ($this->filterLdapAuthenticated && !isset($user->data['ldap_user'])) ||
        ($filterOnRoles && !array_intersect(array_values($selectedRoles), array_keys($user->roles)))
        ) {
        continue;
      }

      $factory = \Drupal::service('ldap.servers');

      if ($ldap_user = $factory->getUserDataByAccount($user)) {
        unset($ldap_user['mail']);
        $ldap_user['attr']['count'] = $ldap_user['attr']['count'] + count($this->availableDrupalUserAttributes);
        foreach ($this->availableDrupalUserAttributes as $attr_name => $attr_conf) {
          $ldap_user['attr'][] = $attr_conf['token'];
          $ldap_user['attr'][$attr_conf['token']]['count'] = 1;
          $ldap_user['attr'][$attr_conf['token']][0] = (string) $user->{$attr_name};
        }

        $results[] = $ldap_user;
      }
    }
    $results['count'] = count($results);
    return $results;
  }

}

/**
 * Fetches data via LDAP Query.
 */
class FeedsDrupalUserLdapEntryFetcher extends FeedsFetcher {

  /**
   * Implements FeedsFetcher::fetch().
   */
  public function fetch(FeedsSource $source) {
    $source_config = $source->getConfigFor($this);
    $result = new FeedsDrupalUserLdapEntryFetcherResult($source_config);
    return $result;
  }

  /**
   * Override parent::configDefaults().
   */
  public function configDefaults() {
    return [
      'filterLdapAuthenticated' => [],
      'availableDrupalUserAttributes' => ldap_feeds_drupal_user_attributes(),
      'filterRoles' => [],
    ];
  }

  /**
   * Override parent::configForm().
   */
  public function configForm(&$form_state) {

    $form = [];
    $form['filterLdapAuthenticated'] = [
      '#type' => 'checkbox',
      '#title' => t('Only return ldap authenticated users.'),
      '#default_value' => $this->config['filterLdapAuthenticated'],
      '#description' => t('If checked, only users who are associated with ldap accounts will be returned.'),
    ];

    return $form;
  }

  /**
   * Override parent::sourceFormValidate().
   */
  public function sourceFormValidate(&$values) {

  }

  /**
   * Override parent::sourceForm().
   */
  public function sourceForm($source_config) {
    $tokens = [
      '%edit_link' => \Drupal::l(t('Edit Feed'), Url::fromUri('/admin/structure/feeds/edit/' . $this->id)),
    ];

    $form_state = [];
    $form = $this->configForm($form_state);
    $form['addendum'] = [
      '#type' => 'markup',
      '#markup' => t('This import is configured at %edit_link.', $tokens),
    ];

    return $form;
  }

}
